module NxtHerokuDatabaseBackup
  module Uploader
    class S3
      include NxtInit

      class << self
        def build(bucket_name:, file_path:, backup_name:, credentials: default_credentials)
          new(
            bucket_name: bucket_name,
            file_path: file_path,
            backup_name: backup_name,
            credentials: credentials
          )
        end

        private

        def default_credentials
          {
            access_key_id: ENV.fetch('DATABASE_BACKUP_AWS_ACCESS_KEY_ID'),
            secret_access_key: ENV.fetch('DATABASE_BACKUP_AWS_SECRET_ACCESS_KEY'),
            region: ENV.fetch('DATABASE_BACKUP_AWS_REGION')
          }
        end
      end

      attr_init :bucket_name, :file_path, :credentials, :backup_name

      def call
        aws_object.upload_file(file_path)
      end

      def aws_object
        @aws_object ||= begin
          ensure_bucket_exists
          Aws::S3::Resource.new(credentials).bucket(bucket_name).object(backup_name)
        end
      end

      def ensure_bucket_exists
        client.head_bucket(bucket: bucket_name)
      rescue Aws::S3::Errors::NotFound
        client.create_bucket(bucket: bucket_name)
      end

      def client
        @client ||= Aws::S3::Client.new(**credentials)
      end
    end
  end
end
